pro read_lunar, filename, status, lunar

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Lunar Event Species Data binary product files.
;
;  Author:
;  Karen H. Sage
;  k.h.sage@larc.nasa.gov
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  08/01/01 - Deleted variable definitions and read of pressure
;             surface variables because of change in file format.
;
;  Input variables:  the name of the lunar product file (the product 
;     file names have the form  g3a.lspb.*).
;  Output variables:  a read file status variable and the structure
;     containing the lunar product data.  See the SAGE III Data Products 
;     Users Guide for a description of the lunar product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50

;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

lunar_size = { lunar_size, event_id:0L, $
          yr_day_tag:0L, $
          mission_time:0., $
          int_fill_value:0L, $
          flt_fill_value:0., $
          mission_id:0L, $
          L0DO_Version:0., $
          L0_Version:0., $
          Director_Version:0., $
          Solar_Version:0., $
          DB_Version:0., $
          GRAM95_Version:0., $
          Met_Version:0., $
          Lun_Model_Ver:0., $
          Lun_Albedo_Ver:0., $
          bin_height:0., $
          num_alt_bins:0L, $
          num_press_grid:0L, $
          num_grnd_trk:0L }


;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.

readu, in_unit, lunar_size

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then lunar_size = swap_endian (lunar_size)

;  Create the structure for read of all data from binary file.


lunar = { lunar, $
     Event_ID:0L, $
     yr_day_tag:0L, $
     mission_time:0., $
     int_fill_value:0L, $
     flt_fill_value:0., $
     mission_id:0L, $
     L0DO_Version:0., $
     L0_Version:0., $
     Director_Version:0., $
     Solar_Version:0., $
     DB_Version:0., $
     GRAM95_Version:0., $
     Met_Version:0., $
     Lun_Model_Ver:0., $
     Lun_Albedo_Ver:0., $
     bin_height:0., $
     num_alt_bins:0L, $
     num_press_grid:0L, $
     num_grnd_trk:0L, $
     sc_evt_type:0L, $
     gnd_evt_type:0L, $
     BetaAngle:0., $
     LunarPhase:0., $
     ZenithAngle:0., $
     aurora_flag:0L, $
     st_date:0L, $
     st_time:0L, $
     st_latitude:0., $
     st_longitude:0., $
     st_altitude:0., $
     end_date:0L, $
     end_time:0L, $
     end_latitude:0., $
     end_longitude:0., $
     end_altitude:0., $
     gt_date:lonarr(lunar_size.num_grnd_trk), $
     gt_time:lonarr(lunar_size.num_grnd_trk), $
     gt_latitude:fltarr(lunar_size.num_grnd_trk), $
     gt_longitude:fltarr(lunar_size.num_grnd_trk), $
     gt_ray_dir:fltarr(lunar_size.num_grnd_trk), $
     trop_temp:0., $
     trop_alt:0., $
     met_pressure:fltarr(lunar_size.num_press_grid), $
     met_temp:fltarr(lunar_size.num_press_grid), $
     met_temp_unc:fltarr(lunar_size.num_press_grid), $
     met_altitude:fltarr(lunar_size.num_press_grid), $
     Met_Source:0L, $
     geom_alt:fltarr(lunar_size.num_alt_bins), $
     AltRegQA:lonarr(lunar_size.num_alt_bins), $
     potential_alt:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp_uncert:fltarr(lunar_size.num_alt_bins), $
     CurrentPress:fltarr(lunar_size.num_alt_bins), $
     CurrentPress_uncert:fltarr(lunar_size.num_alt_bins), $
     TPSource:lonarr(lunar_size.num_alt_bins), $
     CompositeO3:fltarr(lunar_size.num_alt_bins), $
     CompositeO3_uncert:fltarr(lunar_size.num_alt_bins), $
     CompO3QA:lonarr(lunar_size.num_alt_bins), $
     NO2NDP:fltarr(lunar_size.num_alt_bins), $
     NO2NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO2QA:lonarr(lunar_size.num_alt_bins), $
     NO3NDP:fltarr(lunar_size.num_alt_bins), $
     NO3NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO3QA:lonarr(lunar_size.num_alt_bins), $
     OClONDP:fltarr(lunar_size.num_alt_bins), $
     OClONDP_uncert:fltarr(lunar_size.num_alt_bins), $
     OClOQA:lonarr(lunar_size.num_alt_bins) }

;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

readu, in_unit, lunar

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then lunar = swap_endian (lunar)


;  Close data file unit and free the logical unit number.

close, in_unit
free_lun, in_unit
goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:

end
