pro read_solar, filename, status, solar

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Solar Event Species Data binary product files.
;
;  Author:
;  Karen H. Sage
;  k.h.sage@larc.nasa.gov
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  08/01/01 - Deleted variable definitions and read of pressure
;             surface variables because of change in file format.
;
;  Input variable:  the name of the solar product file (the product 
;     file names have the form  g3a.sspb.*)
;  Output variables:  a read file status variable and the structure 
;     containing the solar product data.  See the SAGE III Data
;     Products Users Guide for a description of the solar product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50


;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

solar_size = {solar_size, event_id:0L, $
                 yr_day_tag:0L, $
                 mission_time:0., $
                 int_fill_value:0L, $
                 flt_fill_value:0., $
                 mission_id:0L, $
                 L0DO_Version:0., $
                 L0_Version:0., $
                 Director_version:0., $
                 Solar_version:0., $
                 DB_Version:0., $
                 GRAM95_Version:0., $
                 Met_Version:0., $
                 bin_height:0., $
                 num_bins:0L, $
                 num_ao3_channels:0L,$
                 num_grnd_trk:0L, $
                 num_aer_bins:0L}

;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.

readu, in_unit, solar_size

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_size = swap_endian (solar_size)

;  Create a structure for the aerosol extinction channel data.
;  NOTE:  The tag AerosolExt is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         AerosolExt, the correct syntax is solar.AerosolExt(i).aerext

aerosol_ext = { aerosol_ext, $
                aerext:fltarr(solar_size.num_aer_bins), $
                aerext_uncert:fltarr(solar_size.num_aer_bins), $
                aerQA:lonarr(solar_size.num_aer_bins) }

 

;  Create the structure for read of all data from binary file.


solar = {solar, $ 
      event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Director_version:0., $
      Solar_version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_ao3_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      geom_alt:fltarr(solar_size.num_bins), $
      potential_alt:fltarr(solar_size.num_bins), $
      CurrentTemp:fltarr(solar_size.num_bins), $
      CurrentTemp_uncert:fltarr(solar_size.num_bins), $
      CurrentPress:fltarr(solar_size.num_bins), $
      CurrentPress_uncert:fltarr(solar_size.num_bins), $
      TPSource:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      CompositeO3:fltarr(solar_size.num_bins), $
      CompositeO3_uncert:fltarr(solar_size.num_bins), $
      SPOzone:fltarr(solar_size.num_bins), $
      SPOzone_uncert:fltarr(solar_size.num_bins), $
      CompO3QA:lonarr(solar_size.num_bins), $
      MesO3:fltarr(solar_size.num_bins), $
      MesO3_uncert:fltarr(solar_size.num_bins), $
      MesSPOzone:fltarr(solar_size.num_bins), $
      MesSPOzone_uncert:fltarr(solar_size.num_bins), $
      MesO3QA:lonarr(solar_size.num_bins), $
      ChapO3:fltarr(solar_size.num_bins), $
      ChapO3_uncert:fltarr(solar_size.num_bins), $
      ChapSPOzone:fltarr(solar_size.num_bins), $
      ChapSPOzone_uncert:fltarr(solar_size.num_bins), $
      MLRO3QA:lonarr(solar_size.num_bins), $
      WaterVaporNDP:fltarr(solar_size.num_bins), $
      WaterVaporNDP_uncert:fltarr(solar_size.num_bins), $
      H2OQA:lonarr(solar_size.num_bins), $
      NO2NDP:fltarr(solar_size.num_bins), $
      NO2NDP_uncert:fltarr(solar_size.num_bins), $
      SPNO2:fltarr(solar_size.num_bins), $
      SPNO2_uncert:fltarr(solar_size.num_bins), $
      NO2QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTPQA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_ao3_channels), $
      Aer_width:fltarr(solar_size.num_ao3_channels), $
      StratOD:fltarr(solar_size.num_ao3_channels), $
      StratOD_uncert:fltarr(solar_size.num_ao3_channels), $
      StratODQA:lonarr(solar_size.num_ao3_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_ao3_channels) , $
      asd_flag:lonarr(solar_size.num_aer_bins), $
      Aerosol1020:fltarr(solar_size.num_aer_bins), $
      Aerosol1020_uncert:fltarr(solar_size.num_aer_bins), $
      Aero1020QA:lonarr(solar_size.num_aer_bins) } 
      
;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

readu, in_unit, solar

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar = swap_endian(solar)


close, in_unit
free_lun, in_unit
goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:
end
