pro read_solar_trans, filename, status, solar_trans

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Solar Transmission binary product files.
;
;
;  Author:
;  Karen H. Sage
;  k.h.sage@larc.nasa.gov
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;
;  Input variable:  the name of the solar transmission product file
;     (the product file names have the form  g3a.tb.*)
;  Output variables:  a read file status variable and the structure
;     containing the solar transmission product data.  See the SAGE III
;     Data Products Users Guide for a description of the solar
;     transmission product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50


;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

solar_trans_size = {solar_trans_size, event_id:0L, $
                 yr_day_tag:0L, $
                 mission_time:0., $
                 int_fill_value:0L, $
                 flt_fill_value:0., $
                 mission_id:0L, $
                 L0DO_Version:0., $
                 L0_Version:0., $
                 Director_Version:0., $
                 Solar_Version:0., $
                 DB_Version:0., $
                 GRAM95_Version:0., $
                 Met_Version:0., $
                 bin_height:0., $
                 profile_count:0L, $
                 num_grnd_trk:0L, $
                 num_press_grid:0L, $
                 num_CCDPxlGrps:0L, $
                 num_alt_bins:0L }

;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.

readu, in_unit, solar_trans_size

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_trans_size = swap_endian (solar_trans_size)


;  Create a structure for the pixel group transmission profile.
;  NOTE:  The tag pg_trans_profiles is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         array, an example of the correct syntax is 
;         solar_trans.pg_trans_profiles(i).Transmission

pg_trans_profile = { trans_profile, $
                      Transmission:fltarr(solar_trans_size.num_alt_bins,2), $
                      TransQA:lonarr(solar_trans_size.num_alt_bins) }


;  Create the structure for read of all data from binary file. 
;  NOTE:  Last tag in the structure is an array containing the 
;   pg_trans_profile structure.


solar_trans = {solar_trans, $ 
      event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Director_Version:0., $
      Solar_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      trop_temp:0., $
      trop_alt:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      CurrentPress:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp_uncert:fltarr(solar_trans_size.num_alt_bins), $
      TPSource:lonarr(solar_trans_size.num_alt_bins), $ 
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }


;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

readu, in_unit, solar_trans

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_trans = swap_endian(solar_trans)


close, in_unit
free_lun, in_unit

goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:
end


