pro read_cloud_bin, filename, status, cloud, rec_num

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Solar Event Species Data binary product files.
;
;  Author:
;  Karen H. Sage
;  k.h.sage@larc.nasa.gov
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  08/01/2001 - Deleted variable definitions and read of pressure
;             surface variables because of change in file format.
;  09/01/2004 - Made cloud an unnamed structure, added the number
;             of records as an output parameter.
;
;  Input variable:  the name of the solar product file (the product 
;     file names have the form  g3a.sspb.*)
;  Output variables:  a read file status variable, the structure 
;     containing the solar product data and the number of records
;     in the structure.  See the SAGE III Data Products Users Guide 
;     for a description of the cloud product data. Immediately below
;     is a description of the cloud presence variable.


;***************************************************************************
;   cloud.Cloud_presence is the variable checked for presence of cloud
;     in the monthly cloud product.  Its values are:
;        0  insufficient input data (i.e., data missing for at least
;                                    one aerosol wavelength)
;        1  no cloud present (representative of background upper
;                             troposhperic or stratospheric aerosol)
;        2  no cloud present (representative of volcanically enhanced
;                             aerosols)
;        3  no cloud present (representative of large upper tropospheric
;                             aerosols)
;        4  cloud present
;***************************************************************************

 

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50


;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read number of records.

rec_num = 0L
readu,in_unit,rec_num
if IEEE_flag eq 0 then rec_num = swap_endian(rec_num)

;  Read comment.
letter = ' '
comment = replicate(letter,996)
readu, in_unit, comment

;  Define the structure to hold size information for later definitions.
cloud_size = {cloud_size, $
      event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Director_version:0., $
      Solar_version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      Cloud_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_grnd_trk:0L }

;  Read size structure.
readu, in_unit, cloud_size
if IEEE_flag eq 0 then cloud_size = swap_endian(cloud_size)

;             01234567890123456789012345678901234567890123456789012345678901234567890123456789
;cl_comment = '                                                                                '
cl_comment = replicate(letter,80)
;cl_comment = strarr(5)
cloud_rec = {cloud_rec, $
      event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Director_version:0., $
      Solar_version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      Cloud_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_grnd_trk:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0. , $
      gt_date:lonarr(cloud_size.num_grnd_trk), $
      gt_time:lonarr(cloud_size.num_grnd_trk), $
      gt_latitude:fltarr(cloud_size.num_grnd_trk), $
      gt_longitude:fltarr(cloud_size.num_grnd_trk), $
      gt_ray_dir:fltarr(cloud_size.num_grnd_trk), $
      Cloud_presence:lonarr(cloud_size.num_bins), $
      Cloud_presence_uncert:lonarr(cloud_size.num_bins), $
      Cloud_areas:lonarr(cloud_size.num_bins), $
      Cloud_comment:cl_comment }

cloud = { $
      event_id:lonarr(rec_num), $
      yr_day_tag:lonarr(rec_num), $
      mission_time:fltarr(rec_num), $
      int_fill_value:lonarr(rec_num), $
      flt_fill_value:fltarr(rec_num), $
      mission_id:lonarr(rec_num), $
      L0DO_Version:fltarr(rec_num), $
      L0_Version:fltarr(rec_num), $
      Director_version:fltarr(rec_num), $
      Solar_version:fltarr(rec_num), $
      DB_Version:fltarr(rec_num), $
      GRAM95_Version:fltarr(rec_num), $
      Met_Version:fltarr(rec_num), $
      Cloud_Version:fltarr(rec_num), $
      bin_height:fltarr(rec_num), $
      num_bins:lonarr(rec_num), $
      num_grnd_trk:lonarr(rec_num), $
      sc_evt_type:lonarr(rec_num), $
      gnd_evt_type:lonarr(rec_num), $
      BetaAngle:fltarr(rec_num), $
      st_date:lonarr(rec_num), $
      st_time:lonarr(rec_num), $
      st_latitude:fltarr(rec_num), $
      st_longitude:fltarr(rec_num), $
      st_altitude:fltarr(rec_num), $
      end_date:lonarr(rec_num), $
      end_time:lonarr(rec_num), $
      end_latitude:fltarr(rec_num), $
      end_longitude:fltarr(rec_num), $
      end_altitude:fltarr(rec_num), $
      gt_date:lonarr(rec_num,cloud_size.num_grnd_trk), $
      gt_time:lonarr(rec_num,cloud_size.num_grnd_trk), $
      gt_latitude:fltarr(rec_num,cloud_size.num_grnd_trk), $
      gt_longitude:fltarr(rec_num,cloud_size.num_grnd_trk), $
      gt_ray_dir:fltarr(rec_num,cloud_size.num_grnd_trk), $
      Cloud_presence:intarr(rec_num,cloud_size.num_bins), $
      Cloud_presence_uncert:intarr(rec_num,cloud_size.num_bins), $
      Cloud_areas:intarr(rec_num,cloud_size.num_bins), $
      Cloud_comment:strarr(rec_num,80) }

for i = 0, rec_num-1 do begin      
 
  ;  Change file pointer to beginning of second record if first time through.
  if i eq 0 then point_lun, in_unit, 1000
;  point_lun, -in_unit, pos

  ;  Read one record
  readu, in_unit, cloud_rec

  ;  If machine is not IEEE then swap bytes to make them compliant.
  if IEEE_flag eq 0 then cloud_rec = swap_endian(cloud_rec)

  ;  Save the record in the cloud structure.
  cloud.event_id(i) = cloud_rec.event_id
  cloud.yr_day_tag(i) = cloud_rec.yr_day_tag
  cloud.mission_time(i) = cloud_rec.mission_time
  cloud.int_fill_value(i) = cloud_rec.int_fill_value
  cloud.flt_fill_value(i) = cloud_rec.flt_fill_value
  cloud.mission_id(i) = cloud_rec.mission_id
  cloud.L0DO_Version(i) = cloud_rec.L0DO_Version
  cloud.L0_Version(i) = cloud_rec.L0_Version
  cloud.Director_version(i) = cloud_rec.Director_version
  cloud.Solar_version(i) = cloud_rec.Solar_version
  cloud.DB_Version(i) = cloud_rec.DB_Version
  cloud.GRAM95_Version(i) = cloud_rec.GRAM95_Version
  cloud.Met_Version(i) = cloud_rec.Met_Version
  cloud.Cloud_Version(i) = cloud_rec.Cloud_Version
  cloud.bin_height(i) = cloud_rec.bin_height
  cloud.num_bins(i) = cloud_rec.num_bins
  cloud.num_grnd_trk(i) = cloud_rec.num_grnd_trk
  cloud.sc_evt_type(i) = cloud_rec.sc_evt_type
  cloud.gnd_evt_type(i) = cloud_rec.gnd_evt_type
  cloud.BetaAngle(i) = cloud_rec.BetaAngle
  cloud.st_date(i) = cloud_rec.st_date
  cloud.st_time(i) = cloud_rec.st_time
  cloud.st_latitude(i) = cloud_rec.st_latitude
  cloud.st_longitude(i) = cloud_rec.st_longitude
  cloud.st_altitude(i) = cloud_rec.st_altitude
  cloud.end_date(i) = cloud_rec.end_date
  cloud.end_time(i) = cloud_rec.end_time
  cloud.end_latitude(i) = cloud_rec.end_latitude
  cloud.end_longitude(i) = cloud_rec.end_longitude
  cloud.end_altitude(i) = cloud_rec.end_altitude
  cloud.gt_date(i,*) = cloud_rec.gt_date
  cloud.gt_time(i,*) = cloud_rec.gt_time
  cloud.gt_latitude(i,*) = cloud_rec.gt_latitude
  cloud.gt_longitude(i,*) = cloud_rec.gt_longitude
  cloud.gt_ray_dir(i,*) = cloud_rec.gt_ray_dir
  cloud.Cloud_presence(i,*) = cloud_rec.Cloud_presence
  cloud.Cloud_presence_uncert(i,*) = cloud_rec.Cloud_presence_uncert
  cloud.Cloud_areas(i,*) = cloud_rec.Cloud_areas
  cloud.Cloud_comment(i,*) = cloud_rec.Cloud_comment
endfor

close, in_unit
free_lun, in_unit
goto, program_end

  open_error: 
print, 'Error on open or read of file ', filename
status = 'ERROR'

  program_end:
end

